/*
 *   Copyright 1998-2000 University of Illinois Board of Trustees
 *   Copyright 1998-2000 Mark D. Roth
 *   All rights reserved. 
 *
 *   fget_listhash.h - header file for fget library
 *
 *   Mark D. Roth <roth@uiuc.edu>
 *   Computing and Communications Services Office
 *   University of Illinois at Urbana-Champaign
 */


/***** list.c **********************************************************/

struct fget_node {
  void *data;
  struct fget_node *next;
  struct fget_node *prev;
};
typedef struct fget_node fget_node_t;

struct fget_list {
  fget_node_t *first;
  fget_node_t *last;
  int (*cmpfunc)();	/* called as (*cmpfunc)(void *data1, void *data2)
			   returns less than, equal to, or greater than 0
			   if data1 is <, =, or > data2 */
  int flags;
};
typedef struct fget_list fget_list_t;

/* values for flags */
#define LIST_USERFUNC	0	/* use cmpfunc() to order */
#define LIST_STACK	1	/* new elements go in front */
#define LIST_QUEUE	2	/* new elements go at the end */


/* creates a new, empty list */
fget_list_t *fget_list_new(int flags, int (*cmpfunc)());

/* remove and free() the entire list */
void fget_list_free(fget_list_t *l, void (*freefunc)());

/* add elements */
int fget_list_add(fget_list_t *l, void *data);

/* removes an element from the list - returns -1 on error */
void fget_list_del(fget_list_t *l, fget_node_t **n);

/* returns 1 when valid data is returned, or 0 at end of list */
int fget_list_next(fget_list_t *l, fget_node_t **n);

/* returns 1 when valid data is returned, or 0 at end of list */
int fget_list_prev(fget_list_t *l, fget_node_t **n);

/* copies an existing list */
fget_list_t *fget_list_dup(fget_list_t *l);

/* merges list2 and list1 and returns merged list */
fget_list_t *fget_list_merge(int (*cmpfunc)(), int flags,
                                   fget_list_t *list1,
                                   fget_list_t *list2);

/* return 1 if the data matches a list entry, 0 otherwise */
int fget_list_search(fget_list_t *l, fget_node_t **n,
                        void *data, int (*matchfunc)());

/* adds elements from a string delimited by delim */
int fget_list_add_str(fget_list_t *l, char *str, char *delim);

/* string matching function */
int fget_str_match(char *check, char *data);


/***** hash.c **********************************************************/

typedef struct {
  int bucket;
  fget_node_t *node;
} fget_hashptr_t;

typedef struct {
  int numbuckets;
  fget_list_t **table;
  int (*hashfunc)();
} fget_hash_t;

/* default hash function, optimized for 7-bit strings */
int fget_str_hashfunc(char *key, int numbuckets);

/* create a new hash */
fget_hash_t *fget_hash_new(int num, int (*hashfunc)());

/* delete all the fget_nodes of the hash and clean up */
void fget_hash_free(fget_hash_t *h, void (*freefunc)());

/* returns 1 when valid data is returned, or 0 at end of list */
int fget_hash_next(fget_hash_t *h, fget_hashptr_t *hp);

/* return 1 if the data matches a list entry, 0 otherwise */
int fget_hash_search(fget_hash_t *h, fget_hashptr_t *hp,
                        void *data, int (*matchfunc)());

/* return 1 if the key matches a list entry, 0 otherwise */
int fget_hash_getkey(fget_hash_t *h, fget_hashptr_t *hp,
                        void *key, int (*matchfunc)());

/* inserting data */
int fget_hash_add(fget_hash_t *h, void *data);

/* delete an entry */
int fget_hash_del(fget_hash_t *h, fget_hashptr_t *hp);


