/*
 *   Copyright 2000-2001 University of Illinois Board of Trustees
 *   Copyright 2000-2001 Mark D. Roth
 *   All rights reserved. 
 *
 *   libfget.h - header file for libfget
 *
 *   Mark D. Roth <roth@uiuc.edu>
 *   Computing and Communications Services Office
 *   University of Illinois at Urbana-Champaign
 */

#ifndef LIBFGET_H
#define LIBFGET_H

#include <netdb.h>
#include <sys/types.h>
#include <sys/param.h>

#include <fget_listhash.h>


#define FTPBUFSIZE	10240		/* arbitrary useful length */
#define FTPSYSTYPELEN	40		/* taken from RFC-1340 */

#define MAX_SYMLINK_RECURSION	16

#ifndef MAXUSERNAMELEN
#define MAXUSERNAMELEN	16		/* max length of user name */
#endif

#ifndef MAXGROUPNAMELEN
#define MAXGROUPNAMELEN	8		/* max length of group name */
#endif

/* macros for bitfield manipulation */
#define BIT_ISSET(bitfield, opt) ((bitfield) & (opt))
#define BIT_SET(bitfield, opt) ((bitfield) |= (opt))
#define BIT_CLEAR(bitfield, opt) ((bitfield) &= ~(opt))
#define BIT_TOGGLE(bitfield, opt) { \
  if (BIT_ISSET(bitfield, opt)) \
    BIT_CLEAR(bitfield, opt); \
  else \
    BIT_SET(bitfield, opt); \
}


struct ftpbuf {
  char fb_buf[FTPBUFSIZE];		/* receive buffer */
  char *fb_readp, *fb_writep;		/* buffer pointers */
};
typedef struct ftpbuf FTPBUF;

#define ftp_init_buf(buf) { \
  (buf).fb_readp = (buf).fb_writep = (buf).fb_buf; \
  (buf).fb_buf[0] = '\0'; \
}


/***** ftpstat.c *************************************************************/

struct ftpstat {
  char fs_filename[MAXPATHLEN];		/* file name */
  char fs_linkname[MAXPATHLEN];		/* link name */
  mode_t fs_mode;			/* file mode */
  int fs_refcount;			/* inode reference count */
  char fs_username[MAXUSERNAMELEN];	/* user name */
  char fs_groupname[MAXGROUPNAMELEN];	/* group name */
  size_t fs_size;			/* size */
  time_t fs_date;			/* date */
  fget_hash_t *fs_dir_h;		/* file list for directories */
  int fs_dc_flags;			/* dir cache flags */
  time_t fs_dc_time;			/* dir cache timestamp */
};

/* values for fs_dc_flags bitmask */
#define DC_VALID		1	/* entry is valid (not just stub) */
#define DC_DIR_VALID		2	/* directory entry is valid */
#define DC_DIR_HIDDEN		4	/* directory is hidden */

typedef struct ftp FTP;

/* stat a symlink */
int ftp_lstat(FTP *ftp, char *pathname, struct ftpstat *fsp);

/* stat a file */
int ftp_stat(FTP *ftp, char *pathname, struct ftpstat *fsp);

/* read the contents of a symbolic link */
int ftp_readlink(FTP *ftp, char *pathname, char *buf, int bufsize);

/* resolve symlinks */
int ftp_realpath(FTP *ftp, char *path, char *resolved_path, int resolvelen);


/***** handle.c **************************************************************/

extern const char libfget_version[];

struct ftp {
  int ftp_fd, ftp_fdc, ftp_fdl;		/* file descriptors */
  void (*ftp_sendhook)();		/* debug hook for sending */
  void (*ftp_recvhook)();		/* debug hook for receiving */
  char ftp_host[MAXHOSTNAMELEN + 6];	/* hostname */
  char ftp_dir[MAXPATHLEN];		/* current directory */
  char ftp_login[MAXUSERNAMELEN];	/* remote username */
  char ftp_systype[FTPSYSTYPELEN];	/* remote system type */
  unsigned long ftp_features;		/* server features (see below) */
  unsigned long ftp_flags;		/* caller-selected flags */
  FTPBUF ftp_buf;			/* receive buffer */
  struct ftpstat ftp_cache;		/* directory cache */
  time_t ftp_expire;			/* interval before cache expiration */
};

/* values for ftp_features bitmask */
#define FTP_FEAT		 1	/* supports FEAT and OPTS */
#define FTP_SIZE		 2	/* supports SIZE command */
#define FTP_MDTM		 4	/* supports MDTM command */
#define FTP_REST_STREAM		 8	/* supports REST for STREAM xfers */
#define FTP_TVFS		16	/* supports TVFS presentation */
#define FTP_MLST		32	/* supports MLST and MLSD */
#define FTP_NO_LIST_A		64	/* server doesn't grok "LIST -A". */

/* values for ftp_flags bitmask */
#define FTP_PASSIVE		 1	/* use PASV mode for data connection */
#define FTP_ROUNDROBIN		 2	/* grok round-robin DNS entries */
#define FTP_NODIRCACHE		 4	/* don't cache directory information */


/*
** ftp_connect() - open a control connection to the FTP server
** returns:
**	0				success
**	-1 (and sets errno)		failure
*/
int ftp_connect(FTP **ftp, char *hostname, char *banner, int bannersize,
                int flags, void (*sendhook)(), void (*recvhook)());


/* macros for accessing flags */
#define ftp_get_flags(ftp)		((ftp)->ftp_flags)
#define ftp_set_flags(ftp, flags)	((ftp)->ftp_flags = (flags))


/*
** ftp_login() - login to the FTP server
** returns:
**	0				success
**	-1 (and sets errno)		failure
*/
int ftp_login(FTP *ftp, char *login, char *pass);


/*
** ftp_quit() - disconnect from FTP server
** returns:
**	0				success
**	-1 (and sets errno)		failure
*/
int ftp_quit(FTP *ftp, int flags);

/* flags for ftp_quit() */
#define FTP_FASTCLOSE	1


/*
** ftp_fd() - return file descriptor associated with ftp handle
** returns:
**	file descriptor of ftp control connection
*/
int ftp_fd(FTP *ftp);

/* set dircache expiration interval */
void ftp_set_cache_expiretime(FTP *ftp, time_t expiretime);


/***** protocol.c ************************************************************/

/*
** ftp_set_sendhook() - set the send hook function.
*/
void ftp_set_sendhook(FTP *ftp, void (*sendfunc)());

/*
** ftp_set_recvhook() - set the receive hook function.
*/
void ftp_set_recvhook(FTP *ftp, void (*recvfunc)());


/***** ftpdir.c **************************************************************/

struct ftpdir {
  FTP *fd_ftp;				/* FTP structure */
  fget_hash_t *fd_h;			/* directory list */
  fget_hashptr_t fd_hp;			/* internal pointer */
  unsigned int fd_offset;		/* number of entries scanned so far */
};
typedef struct ftpdir FTPDIR;

struct ftpdirent {
  char *fd_name;			/* file name */
};


/* open directory */
int ftp_opendir(FTPDIR **ftpdir, FTP *ftp, char *dir);

/* read directory */
int ftp_readdir(FTPDIR *ftpdir, struct ftpdirent *ftpdirent);

/* rewind directory */
void ftp_rewinddir(FTPDIR *ftpdir);

/* tell where we are in the directory */
unsigned int ftp_telldir(FTPDIR *ftpdir);

/* seek to a given point in the directory */
void ftp_seekdir(FTPDIR *ftpdir, unsigned int loc);

/* close directory */
void ftp_closedir(FTPDIR *ftpdir);


/***** ftpfile.c *************************************************************/

struct ftpfile {
  FTP *ff_ftp;				/* FTP structure */
  int ff_mode;				/* mode (see below) */
  char ff_cmd[FTPBUFSIZE];		/* FTP command */
  struct ftpstat ff_fsp;		/* file info */
  off_t ff_offset;			/* current file offset */
};
typedef struct ftpfile FTPFILE;


/* open a file via FTP */
int ftp_open(FTPFILE **ftpfile, FTP *ftp, char *file, int mode);

/* values for mode */
#define FTP_READ		1
#define FTP_WRITE		2

/* read a file via FTP */
int ftp_read(FTPFILE *ftpfile, char *buf, int bufsize);

/* write a file via FTP */
int ftp_write(FTPFILE *ftpfile, char *buf, int bufsize);

/* seek to a given offset in an FTP file */
int ftp_lseek(FTPFILE *ftpfile, off_t offset, int whence);

/* close an FTP file */
int ftp_close(FTPFILE *ftpfile);


/***** ftpglob.c *************************************************************/

typedef struct {
        int gl_pathc;           /* Count of total paths so far. */
        int gl_matchc;          /* Count of paths matching pattern. */
        int gl_offs;            /* Reserved at beginning of gl_pathv. */
        int gl_flags;           /* Copy of flags parameter to glob. */
        char **gl_pathv;        /* List of paths matching pattern. */
                                /* Copy of errfunc parameter to glob. */
        int (*gl_errfunc)(const char *, int);
} ftp_glob_t;

/* Flags */
# define FTPGLOB_APPEND	0x0001	/* Append to output from previous call. */
# define FTPGLOB_DOOFFS	0x0002	/* Use gl_offs. */
# define FTPGLOB_ERR	0x0004	/* Return on error. */
# define FTPGLOB_MARK	0x0008	/* Append / to matching directories. */
# define FTPGLOB_NOCHECK 0x0010	/* Return pattern itself if nothing matches. */
# define FTPGLOB_NOSORT	0x0020	/* Don't sort. */
# define FTPGLOB_BRACE	0x0080	/* Expand braces ala csh. */
# define FTPGLOB_MAGCHAR 0x0100	/* Pattern had globbing characters. */
# define FTPGLOB_NOMAGIC 0x0200	/* FTPGLOB_NOCHECK without magic chars (csh). */
# define FTPGLOB_NOESCAPE 0x1000 /* Disable backslash escaping. */

/* Error values returned by glob(3) */
# define FTPGLOB_NOSPACE (-1)	/* Malloc call failed. */
# define FTPGLOB_ABORTED (-2)	/* Unignored error. */
# define FTPGLOB_NOMATCH (-3)	/* No match and FTPGLOB_NOCHECK not set. */


/*
** ftp_glob() - generate pathnames of remote files matching a pattern
** returns:
**	0			success
**	FTPGLOB_NOSPACE		malloc() error
**	FTPGLOB_ABORTED		error (and sets errno)
**	FTPGLOB_NOMATCH		no match and FTPGLOB_NOCHECK not set
*/
int ftp_glob(FTP *ftp, const char *pattern, int flags,
             int (*errfunc)(const char *, int), ftp_glob_t *pglob);

/*
** ftp_globfree() - free memory allocated by ftp_glob()
*/
void ftp_globfree(ftp_glob_t *pglob);


/***** ftpops.c **************************************************************/

/* rename a file */
int ftp_rename(FTP *ftp, char *oldname, char *newname);

/* remove a file */
int ftp_remove(FTP *ftp, char *file);

/* execute a site command */
int ftp_site(FTP *ftp, char *sitecmd, int flags);

#define FTP_DATACONN		1	/* ftp_site() needs data connection */

/* determine the remote system type */
char *ftp_systype(FTP *ftp);

/* get server status */
int ftp_status(FTP *ftp, char *buf, int bufsize);

/* change directory */
int ftp_chdir(FTP *ftp, char *dir);

/* make a directory */
int ftp_mkdir(FTP *ftp, char *dir);

/* remove a directory */
int ftp_rmdir(FTP *ftp, char *dir);

/* get current directory */
char *ftp_getcwd(FTP *ftp);


/***** url.c *****************************************************************/

#define MAXPASSLEN              64
#define MAXURLLEN               (MAXHOSTNAMELEN + MAXUSERNAMELEN + \
                                 MAXPASSLEN + MAXPATHLEN + 9)

struct ftp_url {
  char host[MAXHOSTNAMELEN];
  char user[MAXUSERNAMELEN];
  char pass[MAXPASSLEN];
  char path[MAXPATHLEN];
  FTP *ftp;
};
typedef struct ftp_url FTPURL;

/* parse RFC-1738 style URL and fill in caller-supplied fields */
void ftp_parse_url(char *url, FTPURL *ftpurl);


#endif /* ! LIBFGET_H */

